/**
 * A JavaScript implementation of the SHA family of hashes, as defined in FIPS PUB 180-4 and FIPS PUB 202, as
 * well as the corresponding HMAC implementation as defined in FIPS PUB 198a
 *
 * Copyright 2008-2020 Brian Turek, 1998-2009 Paul Johnston & Contributors
 * Distributed under the BSD License
 * See http://caligatio.github.com/jsSHA/ for more information
 *
 * Two ECMAScript polyfill functions carry the following license:
 *
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
 *
 * THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED,
 * INCLUDING WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
 * MERCHANTABLITY OR NON-INFRINGEMENT.
 *
 * See the Apache Version 2.0 License for specific language governing permissions and limitations under the License.
 */
!function(n,r){"object"==typeof exports&&"undefined"!=typeof module?module.exports=r():"function"==typeof define&&define.amd?define(r):(n=n||self).jsSHA=r()}(this,(function(){"use strict";var n="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";function r(n,r,t,e){var i,o,u,w=r||[0],f=(t=t||0)>>>3,s=-1===e?3:0;for(i=0;i<n.length;i+=1)o=(u=i+f)>>>2,w.length<=o&&w.push(0),w[o]|=n[i]<<8*(s+e*(u%4));return{value:w,binLen:8*n.length+t}}function t(t,e,i){switch(e){case"UTF8":case"UTF16BE":case"UTF16LE":break;default:throw new Error("encoding must be UTF8, UTF16BE, or UTF16LE")}switch(t){case"HEX":return function(n,r,t){return function(n,r,t,e){var i,o,u,w;if(0!=n.length%2)throw new Error("String of HEX type must be in byte increments");var f=r||[0],s=(t=t||0)>>>3,h=-1===e?3:0;for(i=0;i<n.length;i+=2){if(o=parseInt(n.substr(i,2),16),isNaN(o))throw new Error("String of HEX type contains invalid characters");for(u=(w=(i>>>1)+s)>>>2;f.length<=u;)f.push(0);f[u]|=o<<8*(h+e*(w%4))}return{value:f,binLen:4*n.length+t}}(n,r,t,i)};case"TEXT":return function(n,r,t){return function(n,r,t,e,i){var o,u,w,f,s,h,a,c,v=0,A=t||[0],E=(e=e||0)>>>3;if("UTF8"===r)for(a=-1===i?3:0,w=0;w<n.length;w+=1)for(u=[],128>(o=n.charCodeAt(w))?u.push(o):2048>o?(u.push(192|o>>>6),u.push(128|63&o)):55296>o||57344<=o?u.push(224|o>>>12,128|o>>>6&63,128|63&o):(w+=1,o=65536+((1023&o)<<10|1023&n.charCodeAt(w)),u.push(240|o>>>18,128|o>>>12&63,128|o>>>6&63,128|63&o)),f=0;f<u.length;f+=1){for(s=(h=v+E)>>>2;A.length<=s;)A.push(0);A[s]|=u[f]<<8*(a+i*(h%4)),v+=1}else for(a=-1===i?2:0,c="UTF16LE"===r&&1!==i||"UTF16LE"!==r&&1===i,w=0;w<n.length;w+=1){for(o=n.charCodeAt(w),!0===c&&(o=(f=255&o)<<8|o>>>8),s=(h=v+E)>>>2;A.length<=s;)A.push(0);A[s]|=o<<8*(a+i*(h%4)),v+=2}return{value:A,binLen:8*v+e}}(n,e,r,t,i)};case"B64":return function(r,t,e){return function(r,t,e,i){var o,u,w,f,s,h,a=0,c=t||[0],v=(e=e||0)>>>3,A=-1===i?3:0,E=r.indexOf("=");if(-1===r.search(/^[a-zA-Z0-9=+/]+$/))throw new Error("Invalid character in base-64 string");if(r=r.replace(/=/g,""),-1!==E&&E<r.length)throw new Error("Invalid '=' found in base-64 string");for(o=0;o<r.length;o+=4){for(f=r.substr(o,4),w=0,u=0;u<f.length;u+=1)w|=n.indexOf(f.charAt(u))<<18-6*u;for(u=0;u<f.length-1;u+=1){for(s=(h=a+v)>>>2;c.length<=s;)c.push(0);c[s]|=(w>>>16-8*u&255)<<8*(A+i*(h%4)),a+=1}}return{value:c,binLen:8*a+e}}(r,t,e,i)};case"BYTES":return function(n,r,t){return function(n,r,t,e){var i,o,u,w,f=r||[0],s=(t=t||0)>>>3,h=-1===e?3:0;for(o=0;o<n.length;o+=1)i=n.charCodeAt(o),u=(w=o+s)>>>2,f.length<=u&&f.push(0),f[u]|=i<<8*(h+e*(w%4));return{value:f,binLen:8*n.length+t}}(n,r,t,i)};case"ARRAYBUFFER":try{new ArrayBuffer(0)}catch(n){throw new Error("ARRAYBUFFER not supported by this environment")}return function(n,t,e){return function(n,t,e,i){return r(new Uint8Array(n),t,e,i)}(n,t,e,i)};case"UINT8ARRAY":try{new Uint8Array(0)}catch(n){throw new Error("UINT8ARRAY not supported by this environment")}return function(n,t,e){return r(n,t,e,i)};default:throw new Error("format must be HEX, TEXT, B64, BYTES, ARRAYBUFFER, or UINT8ARRAY")}}function e(r,t,e,i){switch(r){case"HEX":return function(n){return function(n,r,t,e){var i,o,u="",w=r/8,f=-1===t?3:0;for(i=0;i<w;i+=1)o=n[i>>>2]>>>8*(f+t*(i%4)),u+="0123456789abcdef".charAt(o>>>4&15)+"0123456789abcdef".charAt(15&o);return e.outputUpper?u.toUpperCase():u}(n,t,e,i)};case"B64":return function(r){return function(r,t,e,i){var o,u,w,f,s,h="",a=t/8,c=-1===e?3:0;for(o=0;o<a;o+=3)for(f=o+1<a?r[o+1>>>2]:0,s=o+2<a?r[o+2>>>2]:0,w=(r[o>>>2]>>>8*(c+e*(o%4))&255)<<16|(f>>>8*(c+e*((o+1)%4))&255)<<8|s>>>8*(c+e*((o+2)%4))&255,u=0;u<4;u+=1)h+=8*o+6*u<=t?n.charAt(w>>>6*(3-u)&63):i.b64Pad;return h}(r,t,e,i)};case"BYTES":return function(n){return function(n,r,t){var e,i,o="",u=r/8,w=-1===t?3:0;for(e=0;e<u;e+=1)i=n[e>>>2]>>>8*(w+t*(e%4))&255,o+=String.fromCharCode(i);return o}(n,t,e)};case"ARRAYBUFFER":try{new ArrayBuffer(0)}catch(n){throw new Error("ARRAYBUFFER not supported by this environment")}return function(n){return function(n,r,t){var e,i=r/8,o=new ArrayBuffer(i),u=new Uint8Array(o),w=-1===t?3:0;for(e=0;e<i;e+=1)u[e]=n[e>>>2]>>>8*(w+t*(e%4))&255;return o}(n,t,e)};case"UINT8ARRAY":try{new Uint8Array(0)}catch(n){throw new Error("UINT8ARRAY not supported by this environment")}return function(n){return function(n,r,t){var e,i=r/8,o=-1===t?3:0,u=new Uint8Array(i);for(e=0;e<i;e+=1)u[e]=n[e>>>2]>>>8*(o+t*(e%4))&255;return u}(n,t,e)};default:throw new Error("format must be HEX, B64, BYTES, ARRAYBUFFER, or UINT8ARRAY")}}var i=[1116352408,1899447441,3049323471,3921009573,961987163,1508970993,2453635748,2870763221,3624381080,310598401,607225278,1426881987,1925078388,2162078206,2614888103,3248222580,3835390401,4022224774,264347078,604807628,770255983,1249150122,1555081692,1996064986,2554220882,2821834349,2952996808,3210313671,3336571891,3584528711,113926993,338241895,666307205,773529912,1294757372,1396182291,1695183700,1986661051,2177026350,2456956037,2730485921,2820302411,3259730800,3345764771,3516065817,3600352804,4094571909,275423344,430227734,506948616,659060556,883997877,958139571,1322822218,1537002063,1747873779,1955562222,2024104815,2227730452,2361852424,2428436474,2756734187,3204031479,3329325298],o=[3238371032,914150663,812702999,4144912697,4290775857,1750603025,1694076839,3204075428],u=[1779033703,3144134277,1013904242,2773480762,1359893119,2600822924,528734635,1541459225],w="Chosen SHA variant is not supported";function f(n){var r={outputUpper:!1,b64Pad:"=",shakeLen:-1},t=n||{};if(r.outputUpper=t.outputUpper||!1,t.b64Pad&&(r.b64Pad=t.b64Pad),t.shakeLen){if(t.shakeLen%8!=0)throw new Error("shakeLen must be a multiple of 8");r.shakeLen=t.shakeLen}if("boolean"!=typeof r.outputUpper)throw new Error("Invalid outputUpper formatting option");if("string"!=typeof r.b64Pad)throw new Error("Invalid b64Pad formatting option");return r}var s=function(){function n(n,r,t){var e=t||{};if(this.t=r,this.i=e.encoding||"UTF8",this.numRounds=e.numRounds||1,isNaN(this.numRounds)||this.numRounds!==parseInt(this.numRounds,10)||1>this.numRounds)throw new Error("numRounds must a integer >= 1");this.o=n,this.u=[],this.s=0,this.h=!1,this.v=0,this.A=!1,this.H=[],this.l=[]}return n.prototype.update=function(n){var r,t=0,e=this.S>>>5,i=this.p(n,this.u,this.s),o=i.binLen,u=i.value,w=o>>>5;for(r=0;r<w;r+=e)t+this.S<=o&&(this.R=this.U(u.slice(r,r+e),this.R),t+=this.S);this.v+=t,this.u=u.slice(t>>>5),this.s=o%this.S,this.h=!0},n.prototype.getHash=function(n,r){var t,i,o=this.T;if(!0===this.A)throw new Error("Cannot call getHash after setting HMAC key");var u=f(r);if(!0===this.F){if(-1===u.shakeLen)throw new Error("shakeLen must be specified in options");o=u.shakeLen}var w=e(n,o,this.m,u);for(i=this.B(this.u.slice(),this.s,this.v,this.g(this.R),o),t=1;t<this.numRounds;t+=1)!0===this.F&&o%32!=0&&(i[i.length-1]&=16777215>>>24-o%32),i=this.B(i,o,0,this.Y(this.o),o);return w(i)},n.prototype.setHMACKey=function(n,r,e){var i,o;if(!0===this.A)throw new Error("HMAC key already set");if(!0===this.h)throw new Error("Cannot set HMAC key after calling update");if(!0===this.F)throw new Error("SHAKE is not supported for HMAC");var u=t(r,(e||{}).encoding||"UTF8",this.m)(n),w=u.binLen,f=this.S>>>3,s=f/4-1;for(i=u.value,f<w/8&&(i=this.B(i,w,0,this.Y(this.o),this.T));i.length<=s;)i.push(0);for(o=0;o<=s;o+=1)this.H[o]=909522486^i[o],this.l[o]=1549556828^i[o];this.R=this.U(this.H,this.R),this.v=this.S,this.A=!0},n.prototype.getHMAC=function(n,r){var t;if(!1===this.A)throw new Error("Cannot call getHMAC without first setting HMAC key");var i=f(r),o=e(n,this.T,this.m,i),u=this.B(this.u.slice(),this.s,this.v,this.g(this.R),this.T);return t=this.U(this.l,this.Y(this.o)),o(t=this.B(u,this.T,this.S,t,this.T))},n}(),h=function(n,r){return(h=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(n,r){n.__proto__=r}||function(n,r){for(var t in r)r.hasOwnProperty(t)&&(n[t]=r[t])})(n,r)};function a(n,r){function t(){this.constructor=n}h(n,r),n.prototype=null===r?Object.create(r):(t.prototype=r.prototype,new t)}function c(n,r){return n<<r|n>>>32-r}function v(n,r){return n>>>r|n<<32-r}function A(n,r){return n>>>r}function E(n,r,t){return n^r^t}function H(n,r,t){return n&r^~n&t}function l(n,r,t){return n&r^n&t^r&t}function S(n){return v(n,2)^v(n,13)^v(n,22)}function p(n,r){var t=(65535&n)+(65535&r);return(65535&(n>>>16)+(r>>>16)+(t>>>16))<<16|65535&t}function d(n,r,t,e){var i=(65535&n)+(65535&r)+(65535&t)+(65535&e);return(65535&(n>>>16)+(r>>>16)+(t>>>16)+(e>>>16)+(i>>>16))<<16|65535&i}function R(n,r,t,e,i){var o=(65535&n)+(65535&r)+(65535&t)+(65535&e)+(65535&i);return(65535&(n>>>16)+(r>>>16)+(t>>>16)+(e>>>16)+(i>>>16)+(o>>>16))<<16|65535&o}function y(n){return v(n,7)^v(n,18)^A(n,3)}function U(n){return v(n,6)^v(n,11)^v(n,25)}function b(n){return[1732584193,4023233417,2562383102,271733878,3285377520]}function T(n,r){var t,e,i,o,u,w,f,s=[];for(t=r[0],e=r[1],i=r[2],o=r[3],u=r[4],f=0;f<80;f+=1)s[f]=f<16?n[f]:c(s[f-3]^s[f-8]^s[f-14]^s[f-16],1),w=f<20?R(c(t,5),H(e,i,o),u,1518500249,s[f]):f<40?R(c(t,5),E(e,i,o),u,1859775393,s[f]):f<60?R(c(t,5),l(e,i,o),u,2400959708,s[f]):R(c(t,5),E(e,i,o),u,3395469782,s[f]),u=o,o=i,i=c(e,30),e=t,t=w;return r[0]=p(t,r[0]),r[1]=p(e,r[1]),r[2]=p(i,r[2]),r[3]=p(o,r[3]),r[4]=p(u,r[4]),r}function F(n,r,t,e){for(var i,o=15+(r+65>>>9<<4),u=r+t;n.length<=o;)n.push(0);for(n[r>>>5]|=128<<24-r%32,n[o]=4294967295&u,n[o-1]=u/4294967296|0,i=0;i<n.length;i+=16)e=T(n.slice(i,i+16),e);return e}var m=function(n){function r(r,e,i){var o=this;if("SHA-1"!==r)throw new Error(w);return(o=n.call(this,r,e,i)||this).m=-1,o.p=t(o.t,o.i,o.m),o.U=T,o.g=function(n){return n.slice()},o.Y=b,o.B=F,o.R=[1732584193,4023233417,2562383102,271733878,3285377520],o.S=512,o.T=160,o.F=!1,o}return a(r,n),r}(s);function B(n){return"SHA-224"==n?o.slice():u.slice()}function g(n,r){var t,e,o,u,w,f,s,h,a,c,E,b,T=[];for(t=r[0],e=r[1],o=r[2],u=r[3],w=r[4],f=r[5],s=r[6],h=r[7],E=0;E<64;E+=1)T[E]=E<16?n[E]:d(v(b=T[E-2],17)^v(b,19)^A(b,10),T[E-7],y(T[E-15]),T[E-16]),a=R(h,U(w),H(w,f,s),i[E],T[E]),c=p(S(t),l(t,e,o)),h=s,s=f,f=w,w=p(u,a),u=o,o=e,e=t,t=p(a,c);return r[0]=p(t,r[0]),r[1]=p(e,r[1]),r[2]=p(o,r[2]),r[3]=p(u,r[3]),r[4]=p(w,r[4]),r[5]=p(f,r[5]),r[6]=p(s,r[6]),r[7]=p(h,r[7]),r}var Y=function(n){function r(r,e,i){var o=n.call(this,r,e,i)||this;if(!1==("SHA-224"===r||"SHA-256"===r))throw new Error(w);return o.m=-1,o.p=t(o.t,o.i,o.m),o.U=g,o.g=function(n){return n.slice()},o.Y=B,o.B=function(n,t,e,i){return function(n,r,t,e,i){for(var o,u=15+(r+65>>>9<<4),w=r+t;n.length<=u;)n.push(0);for(n[r>>>5]|=128<<24-r%32,n[u]=4294967295&w,n[u-1]=w/4294967296|0,o=0;o<n.length;o+=16)e=g(n.slice(o,o+16),e);return"SHA-224"===i?[e[0],e[1],e[2],e[3],e[4],e[5],e[6]]:e}(n,t,e,i,r)},o.R=B(r),o.S=512,o.T="SHA-224"===r?224:256,o.F=!1,o}return a(r,n),r}(s),I=function(n,r){this.I=n,this.L=r};function L(n,r){var t;return r>32?(t=64-r,new I(n.L<<r|n.I>>>t,n.I<<r|n.L>>>t)):0!==r?(t=32-r,new I(n.I<<r|n.L>>>t,n.L<<r|n.I>>>t)):n}function C(n,r){var t;return r<32?(t=32-r,new I(n.I>>>r|n.L<<t,n.L>>>r|n.I<<t)):(t=64-r,new I(n.L>>>r|n.I<<t,n.I>>>r|n.L<<t))}function N(n,r){return new I(n.I>>>r,n.L>>>r|n.I<<32-r)}function k(n,r,t){return new I(n.I&r.I^~n.I&t.I,n.L&r.L^~n.L&t.L)}function X(n,r,t){return new I(n.I&r.I^n.I&t.I^r.I&t.I,n.L&r.L^n.L&t.L^r.L&t.L)}function M(n){var r=C(n,28),t=C(n,34),e=C(n,39);return new I(r.I^t.I^e.I,r.L^t.L^e.L)}function K(n,r){var t,e;t=(65535&n.L)+(65535&r.L);var i=(65535&(e=(n.L>>>16)+(r.L>>>16)+(t>>>16)))<<16|65535&t;return t=(65535&n.I)+(65535&r.I)+(e>>>16),e=(n.I>>>16)+(r.I>>>16)+(t>>>16),new I((65535&e)<<16|65535&t,i)}function j(n,r,t,e){var i,o;i=(65535&n.L)+(65535&r.L)+(65535&t.L)+(65535&e.L);var u=(65535&(o=(n.L>>>16)+(r.L>>>16)+(t.L>>>16)+(e.L>>>16)+(i>>>16)))<<16|65535&i;return i=(65535&n.I)+(65535&r.I)+(65535&t.I)+(65535&e.I)+(o>>>16),o=(n.I>>>16)+(r.I>>>16)+(t.I>>>16)+(e.I>>>16)+(i>>>16),new I((65535&o)<<16|65535&i,u)}function _(n,r,t,e,i){var o,u;o=(65535&n.L)+(65535&r.L)+(65535&t.L)+(65535&e.L)+(65535&i.L);var w=(65535&(u=(n.L>>>16)+(r.L>>>16)+(t.L>>>16)+(e.L>>>16)+(i.L>>>16)+(o>>>16)))<<16|65535&o;return o=(65535&n.I)+(65535&r.I)+(65535&t.I)+(65535&e.I)+(65535&i.I)+(u>>>16),u=(n.I>>>16)+(r.I>>>16)+(t.I>>>16)+(e.I>>>16)+(i.I>>>16)+(o>>>16),new I((65535&u)<<16|65535&o,w)}function O(n,r){return new I(n.I^r.I,n.L^r.L)}function P(n){var r=C(n,1),t=C(n,8),e=N(n,7);return new I(r.I^t.I^e.I,r.L^t.L^e.L)}function x(n){var r=C(n,14),t=C(n,18),e=C(n,41);return new I(r.I^t.I^e.I,r.L^t.L^e.L)}var z=[new I(i[0],3609767458),new I(i[1],602891725),new I(i[2],3964484399),new I(i[3],2173295548),new I(i[4],4081628472),new I(i[5],3053834265),new I(i[6],2937671579),new I(i[7],3664609560),new I(i[8],2734883394),new I(i[9],1164996542),new I(i[10],1323610764),new I(i[11],3590304994),new I(i[12],4068182383),new I(i[13],991336113),new I(i[14],633803317),new I(i[15],3479774868),new I(i[16],2666613458),new I(i[17],944711139),new I(i[18],2341262773),new I(i[19],2007800933),new I(i[20],1495990901),new I(i[21],1856431235),new I(i[22],3175218132),new I(i[23],2198950837),new I(i[24],3999719339),new I(i[25],766784016),new I(i[26],2566594879),new I(i[27],3203337956),new I(i[28],1034457026),new I(i[29],2466948901),new I(i[30],3758326383),new I(i[31],168717936),new I(i[32],1188179964),new I(i[33],1546045734),new I(i[34],1522805485),new I(i[35],2643833823),new I(i[36],2343527390),new I(i[37],1014477480),new I(i[38],1206759142),new I(i[39],344077627),new I(i[40],1290863460),new I(i[41],3158454273),new I(i[42],3505952657),new I(i[43],106217008),new I(i[44],3606008344),new I(i[45],1432725776),new I(i[46],1467031594),new I(i[47],851169720),new I(i[48],3100823752),new I(i[49],1363258195),new I(i[50],3750685593),new I(i[51],3785050280),new I(i[52],3318307427),new I(i[53],3812723403),new I(i[54],2003034995),new I(i[55],3602036899),new I(i[56],1575990012),new I(i[57],1125592928),new I(i[58],2716904306),new I(i[59],442776044),new I(i[60],593698344),new I(i[61],3733110249),new I(i[62],2999351573),new I(i[63],3815920427),new I(3391569614,3928383900),new I(3515267271,566280711),new I(3940187606,3454069534),new I(4118630271,4000239992),new I(116418474,1914138554),new I(174292421,2731055270),new I(289380356,3203993006),new I(460393269,320620315),new I(685471733,587496836),new I(852142971,1086792851),new I(1017036298,365543100),new I(1126000580,2618297676),new I(1288033470,3409855158),new I(1501505948,4234509866),new I(1607167915,987167468),new I(1816402316,1246189591)];function Z(n){return"SHA-384"===n?[new I(3418070365,o[0]),new I(1654270250,o[1]),new I(2438529370,o[2]),new I(355462360,o[3]),new I(1731405415,o[4]),new I(41048885895,o[5]),new I(3675008525,o[6]),new I(1203062813,o[7])]:[new I(u[0],4089235720),new I(u[1],2227873595),new I(u[2],4271175723),new I(u[3],1595750129),new I(u[4],2917565137),new I(u[5],725511199),new I(u[6],4215389547),new I(u[7],327033209)]}function q(n,r){var t,e,i,o,u,w,f,s,h,a,c,v,A,E,H,l,S=[];for(t=r[0],e=r[1],i=r[2],o=r[3],u=r[4],w=r[5],f=r[6],s=r[7],c=0;c<80;c+=1)c<16?(v=2*c,S[c]=new I(n[v],n[v+1])):S[c]=j((A=S[c-2],E=void 0,H=void 0,l=void 0,E=C(A,19),H=C(A,61),l=N(A,6),new I(E.I^H.I^l.I,E.L^H.L^l.L)),S[c-7],P(S[c-15]),S[c-16]),h=_(s,x(u),k(u,w,f),z[c],S[c]),a=K(M(t),X(t,e,i)),s=f,f=w,w=u,u=K(o,h),o=i,i=e,e=t,t=K(h,a);return r[0]=K(t,r[0]),r[1]=K(e,r[1]),r[2]=K(i,r[2]),r[3]=K(o,r[3]),r[4]=K(u,r[4]),r[5]=K(w,r[5]),r[6]=K(f,r[6]),r[7]=K(s,r[7]),r}var D=function(n){function r(r,e,i){var o=n.call(this,r,e,i)||this;if(!1==("SHA-384"===r||"SHA-512"===r))throw new Error(w);return o.m=-1,o.p=t(o.t,o.i,o.m),o.U=q,o.g=function(n){return n.slice()},o.Y=Z,o.B=function(n,t,e,i){return function(n,r,t,e,i){for(var o,u=31+(r+129>>>10<<5),w=r+t;n.length<=u;)n.push(0);for(n[r>>>5]|=128<<24-r%32,n[u]=4294967295&w,n[u-1]=w/4294967296|0,o=0;o<n.length;o+=32)e=q(n.slice(o,o+32),e);return"SHA-384"===i?[(e=e)[0].I,e[0].L,e[1].I,e[1].L,e[2].I,e[2].L,e[3].I,e[3].L,e[4].I,e[4].L,e[5].I,e[5].L]:[e[0].I,e[0].L,e[1].I,e[1].L,e[2].I,e[2].L,e[3].I,e[3].L,e[4].I,e[4].L,e[5].I,e[5].L,e[6].I,e[6].L,e[7].I,e[7].L]}(n,t,e,i,r)},o.R=Z(r),o.S=1024,o.T="SHA-384"===r?384:512,o.F=!1,o}return a(r,n),r}(s),G=[new I(0,1),new I(0,32898),new I(2147483648,32906),new I(2147483648,2147516416),new I(0,32907),new I(0,2147483649),new I(2147483648,2147516545),new I(2147483648,32777),new I(0,138),new I(0,136),new I(0,2147516425),new I(0,2147483658),new I(0,2147516555),new I(2147483648,139),new I(2147483648,32905),new I(2147483648,32771),new I(2147483648,32770),new I(2147483648,128),new I(0,32778),new I(2147483648,2147483658),new I(2147483648,2147516545),new I(2147483648,32896),new I(0,2147483649),new I(2147483648,2147516424)],J=[[0,36,3,41,18],[1,44,10,45,2],[62,6,43,15,61],[28,55,25,21,56],[27,20,39,8,14]];function Q(n){var r,t=[];for(r=0;r<5;r+=1)t[r]=[new I(0,0),new I(0,0),new I(0,0),new I(0,0),new I(0,0)];return t}function V(n){var r,t=[];for(r=0;r<5;r+=1)t[r]=n[r].slice();return t}function W(n,r){var t,e,i,o,u,w,f,s,h,a=[],c=[];if(null!==n)for(e=0;e<n.length;e+=2)r[(e>>>1)%5][(e>>>1)/5|0]=O(r[(e>>>1)%5][(e>>>1)/5|0],new I(n[e+1],n[e]));for(t=0;t<24;t+=1){for(o=Q(),e=0;e<5;e+=1)a[e]=(u=r[e][0],w=r[e][1],f=r[e][2],s=r[e][3],h=r[e][4],new I(u.I^w.I^f.I^s.I^h.I,u.L^w.L^f.L^s.L^h.L));for(e=0;e<5;e+=1)c[e]=O(a[(e+4)%5],L(a[(e+1)%5],1));for(e=0;e<5;e+=1)for(i=0;i<5;i+=1)r[e][i]=O(r[e][i],c[e]);for(e=0;e<5;e+=1)for(i=0;i<5;i+=1)o[i][(2*e+3*i)%5]=L(r[e][i],J[e][i]);for(e=0;e<5;e+=1)for(i=0;i<5;i+=1)r[e][i]=O(o[e][i],new I(~o[(e+1)%5][i].I&o[(e+2)%5][i].I,~o[(e+1)%5][i].L&o[(e+2)%5][i].L));r[0][0]=O(r[0][0],G[t])}return r}var $=function(n){function r(r,e,i){var o=this,u=6,f=0;if((o=n.call(this,r,e,i)||this).F=!1,"SHA3-224"===r)f=1152,o.T=224;else if("SHA3-256"===r)f=1088,o.T=256;else if("SHA3-384"===r)f=832,o.T=384;else if("SHA3-512"===r)f=576,o.T=512;else if("SHAKE128"===r)u=31,f=1344,o.T=-1,o.F=!0;else{if("SHAKE256"!==r)throw new Error(w);u=31,f=1088,o.T=-1,o.F=!0}return o.S=f,o.m=1,o.p=t(o.t,o.i,o.m),o.U=W,o.g=V,o.Y=Q,o.B=function(n,r,t,e,i){return function(n,r,t,e,i,o,u){var w,f,s=0,h=[],a=i>>>5,c=r>>>5;for(w=0;w<c&&r>=i;w+=a)e=W(n.slice(w,w+a),e),r-=i;for(n=n.slice(w),r%=i;n.length<a;)n.push(0);for(n[(w=r>>>3)>>2]^=o<<w%4*8,n[a-1]^=2147483648,e=W(n,e);32*h.length<u&&(f=e[s%5][s/5|0],h.push(f.L),!(32*h.length>=u));)h.push(f.I),0==64*(s+=1)%i&&(W(null,e),s=0);return h}(n,r,0,e,f,u,i)},o.R=Q(),o}return a(r,n),r}(s);return function(){function n(n,r,t){if("SHA-1"==n)this.C=new m(n,r,t);else if("SHA-224"==n||"SHA-256"==n)this.C=new Y(n,r,t);else if("SHA-384"==n||"SHA-512"==n)this.C=new D(n,r,t);else{if("SHA3-224"!=n&&"SHA3-256"!=n&&"SHA3-384"!=n&&"SHA3-512"!=n&&"SHAKE128"!=n&&"SHAKE256"!=n)throw new Error(w);this.C=new $(n,r,t)}}return n.prototype.update=function(n){this.C.update(n)},n.prototype.getHash=function(n,r){return this.C.getHash(n,r)},n.prototype.setHMACKey=function(n,r,t){this.C.setHMACKey(n,r,t)},n.prototype.getHMAC=function(n,r){return this.C.getHMAC(n,r)},n}()}));
//# sourceMappingURL=sha.js.map
