/**
 * A JavaScript implementation of the SHA family of hashes, as defined in FIPS PUB 180-4 and FIPS PUB 202, as
 * well as the corresponding HMAC implementation as defined in FIPS PUB 198a
 *
 * Copyright 2008-2020 Brian Turek, 1998-2009 Paul Johnston & Contributors
 * Distributed under the BSD License
 * See http://caligatio.github.com/jsSHA/ for more information
 */
const n="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";function t(n,t,e,r){let i,s,o;const w=t||[0],u=(e=e||0)>>>3,h=-1===r?3:0;for(i=0;i<n.length;i+=1)o=i+u,s=o>>>2,w.length<=s&&w.push(0),w[s]|=n[i]<<8*(h+r*(o%4));return{value:w,binLen:8*n.length+e}}function e(e,r,i){switch(r){case"UTF8":case"UTF16BE":case"UTF16LE":break;default:throw new Error("encoding must be UTF8, UTF16BE, or UTF16LE")}switch(e){case"HEX":return function(n,t,e){return function(n,t,e,r){let i,s,o,w;if(0!=n.length%2)throw new Error("String of HEX type must be in byte increments");const u=t||[0],h=(e=e||0)>>>3,f=-1===r?3:0;for(i=0;i<n.length;i+=2){if(s=parseInt(n.substr(i,2),16),isNaN(s))throw new Error("String of HEX type contains invalid characters");for(w=(i>>>1)+h,o=w>>>2;u.length<=o;)u.push(0);u[o]|=s<<8*(f+r*(w%4))}return{value:u,binLen:4*n.length+e}}(n,t,e,i)};case"TEXT":return function(n,t,e){return function(n,t,e,r,i){let s,o,w,u,h,f,c,a,l=0;const A=e||[0],E=(r=r||0)>>>3;if("UTF8"===t)for(c=-1===i?3:0,w=0;w<n.length;w+=1)for(s=n.charCodeAt(w),o=[],128>s?o.push(s):2048>s?(o.push(192|s>>>6),o.push(128|63&s)):55296>s||57344<=s?o.push(224|s>>>12,128|s>>>6&63,128|63&s):(w+=1,s=65536+((1023&s)<<10|1023&n.charCodeAt(w)),o.push(240|s>>>18,128|s>>>12&63,128|s>>>6&63,128|63&s)),u=0;u<o.length;u+=1){for(f=l+E,h=f>>>2;A.length<=h;)A.push(0);A[h]|=o[u]<<8*(c+i*(f%4)),l+=1}else for(c=-1===i?2:0,a="UTF16LE"===t&&1!==i||"UTF16LE"!==t&&1===i,w=0;w<n.length;w+=1){for(s=n.charCodeAt(w),!0===a&&(u=255&s,s=u<<8|s>>>8),f=l+E,h=f>>>2;A.length<=h;)A.push(0);A[h]|=s<<8*(c+i*(f%4)),l+=2}return{value:A,binLen:8*l+r}}(n,r,t,e,i)};case"B64":return function(t,e,r){return function(t,e,r,i){let s,o,w,u,h,f,c,a=0;const l=e||[0],A=(r=r||0)>>>3,E=-1===i?3:0,H=t.indexOf("=");if(-1===t.search(/^[a-zA-Z0-9=+/]+$/))throw new Error("Invalid character in base-64 string");if(t=t.replace(/=/g,""),-1!==H&&H<t.length)throw new Error("Invalid '=' found in base-64 string");for(o=0;o<t.length;o+=4){for(h=t.substr(o,4),u=0,w=0;w<h.length;w+=1)s=n.indexOf(h.charAt(w)),u|=s<<18-6*w;for(w=0;w<h.length-1;w+=1){for(c=a+A,f=c>>>2;l.length<=f;)l.push(0);l[f]|=(u>>>16-8*w&255)<<8*(E+i*(c%4)),a+=1}}return{value:l,binLen:8*a+r}}(t,e,r,i)};case"BYTES":return function(n,t,e){return function(n,t,e,r){let i,s,o,w;const u=t||[0],h=(e=e||0)>>>3,f=-1===r?3:0;for(s=0;s<n.length;s+=1)i=n.charCodeAt(s),w=s+h,o=w>>>2,u.length<=o&&u.push(0),u[o]|=i<<8*(f+r*(w%4));return{value:u,binLen:8*n.length+e}}(n,t,e,i)};case"ARRAYBUFFER":try{new ArrayBuffer(0)}catch(n){throw new Error("ARRAYBUFFER not supported by this environment")}return function(n,e,r){return function(n,e,r,i){return t(new Uint8Array(n),e,r,i)}(n,e,r,i)};case"UINT8ARRAY":try{new Uint8Array(0)}catch(n){throw new Error("UINT8ARRAY not supported by this environment")}return function(n,e,r){return t(n,e,r,i)};default:throw new Error("format must be HEX, TEXT, B64, BYTES, ARRAYBUFFER, or UINT8ARRAY")}}function r(t,e,r,i){switch(t){case"HEX":return function(n){return function(n,t,e,r){let i,s,o="";const w=t/8,u=-1===e?3:0;for(i=0;i<w;i+=1)s=n[i>>>2]>>>8*(u+e*(i%4)),o+="0123456789abcdef".charAt(s>>>4&15)+"0123456789abcdef".charAt(15&s);return r.outputUpper?o.toUpperCase():o}(n,e,r,i)};case"B64":return function(t){return function(t,e,r,i){let s,o,w,u,h,f="";const c=e/8,a=-1===r?3:0;for(s=0;s<c;s+=3)for(u=s+1<c?t[s+1>>>2]:0,h=s+2<c?t[s+2>>>2]:0,w=(t[s>>>2]>>>8*(a+r*(s%4))&255)<<16|(u>>>8*(a+r*((s+1)%4))&255)<<8|h>>>8*(a+r*((s+2)%4))&255,o=0;o<4;o+=1)f+=8*s+6*o<=e?n.charAt(w>>>6*(3-o)&63):i.b64Pad;return f}(t,e,r,i)};case"BYTES":return function(n){return function(n,t,e){let r,i,s="";const o=t/8,w=-1===e?3:0;for(r=0;r<o;r+=1)i=n[r>>>2]>>>8*(w+e*(r%4))&255,s+=String.fromCharCode(i);return s}(n,e,r)};case"ARRAYBUFFER":try{new ArrayBuffer(0)}catch(n){throw new Error("ARRAYBUFFER not supported by this environment")}return function(n){return function(n,t,e){let r;const i=t/8,s=new ArrayBuffer(i),o=new Uint8Array(s),w=-1===e?3:0;for(r=0;r<i;r+=1)o[r]=n[r>>>2]>>>8*(w+e*(r%4))&255;return s}(n,e,r)};case"UINT8ARRAY":try{new Uint8Array(0)}catch(n){throw new Error("UINT8ARRAY not supported by this environment")}return function(n){return function(n,t,e){let r;const i=t/8,s=-1===e?3:0,o=new Uint8Array(i);for(r=0;r<i;r+=1)o[r]=n[r>>>2]>>>8*(s+e*(r%4))&255;return o}(n,e,r)};default:throw new Error("format must be HEX, B64, BYTES, ARRAYBUFFER, or UINT8ARRAY")}}const i=[1116352408,1899447441,3049323471,3921009573,961987163,1508970993,2453635748,2870763221,3624381080,310598401,607225278,1426881987,1925078388,2162078206,2614888103,3248222580,3835390401,4022224774,264347078,604807628,770255983,1249150122,1555081692,1996064986,2554220882,2821834349,2952996808,3210313671,3336571891,3584528711,113926993,338241895,666307205,773529912,1294757372,1396182291,1695183700,1986661051,2177026350,2456956037,2730485921,2820302411,3259730800,3345764771,3516065817,3600352804,4094571909,275423344,430227734,506948616,659060556,883997877,958139571,1322822218,1537002063,1747873779,1955562222,2024104815,2227730452,2361852424,2428436474,2756734187,3204031479,3329325298],s=[3238371032,914150663,812702999,4144912697,4290775857,1750603025,1694076839,3204075428],o=[1779033703,3144134277,1013904242,2773480762,1359893119,2600822924,528734635,1541459225],w="Chosen SHA variant is not supported";function u(n){const t={outputUpper:!1,b64Pad:"=",shakeLen:-1},e=n||{};if(t.outputUpper=e.outputUpper||!1,e.b64Pad&&(t.b64Pad=e.b64Pad),e.shakeLen){if(e.shakeLen%8!=0)throw new Error("shakeLen must be a multiple of 8");t.shakeLen=e.shakeLen}if("boolean"!=typeof t.outputUpper)throw new Error("Invalid outputUpper formatting option");if("string"!=typeof t.b64Pad)throw new Error("Invalid b64Pad formatting option");return t}class h{constructor(n,t,e){const r=e||{};if(this.t=t,this.i=r.encoding||"UTF8",this.numRounds=r.numRounds||1,isNaN(this.numRounds)||this.numRounds!==parseInt(this.numRounds,10)||1>this.numRounds)throw new Error("numRounds must a integer >= 1");this.s=n,this.o=[],this.u=0,this.h=!1,this.l=0,this.A=!1,this.H=[],this.p=[]}update(n){let t,e=0;const r=this.S>>>5,i=this.R(n,this.o,this.u),s=i.binLen,o=i.value,w=s>>>5;for(t=0;t<w;t+=r)e+this.S<=s&&(this.U=this.T(o.slice(t,t+r),this.U),e+=this.S);this.l+=e,this.o=o.slice(e>>>5),this.u=s%this.S,this.h=!0}getHash(n,t){let e,i,s=this.F;if(!0===this.A)throw new Error("Cannot call getHash after setting HMAC key");const o=u(t);if(!0===this.g){if(-1===o.shakeLen)throw new Error("shakeLen must be specified in options");s=o.shakeLen}const w=r(n,s,this.B,o);for(i=this.m(this.o.slice(),this.u,this.l,this.v(this.U),s),e=1;e<this.numRounds;e+=1)!0===this.g&&s%32!=0&&(i[i.length-1]&=16777215>>>24-s%32),i=this.m(i,s,0,this.Y(this.s),s);return w(i)}setHMACKey(n,t,r){let i,s;if(!0===this.A)throw new Error("HMAC key already set");if(!0===this.h)throw new Error("Cannot set HMAC key after calling update");if(!0===this.g)throw new Error("SHAKE is not supported for HMAC");const o=e(t,(r||{}).encoding||"UTF8",this.B)(n),w=o.binLen,u=this.S>>>3,h=u/4-1;for(i=o.value,u<w/8&&(i=this.m(i,w,0,this.Y(this.s),this.F));i.length<=h;)i.push(0);for(s=0;s<=h;s+=1)this.H[s]=909522486^i[s],this.p[s]=1549556828^i[s];this.U=this.T(this.H,this.U),this.l=this.S,this.A=!0}getHMAC(n,t){let e;if(!1===this.A)throw new Error("Cannot call getHMAC without first setting HMAC key");const i=u(t),s=r(n,this.F,this.B,i),o=this.m(this.o.slice(),this.u,this.l,this.v(this.U),this.F);return e=this.T(this.p,this.Y(this.s)),e=this.m(o,this.F,this.S,e,this.F),s(e)}}function f(n,t){return n<<t|n>>>32-t}function c(n,t){return n>>>t|n<<32-t}function a(n,t){return n>>>t}function l(n,t,e){return n^t^e}function A(n,t,e){return n&t^~n&e}function E(n,t,e){return n&t^n&e^t&e}function H(n){return c(n,2)^c(n,13)^c(n,22)}function p(n,t){const e=(65535&n)+(65535&t);return(65535&(n>>>16)+(t>>>16)+(e>>>16))<<16|65535&e}function S(n,t,e,r){const i=(65535&n)+(65535&t)+(65535&e)+(65535&r);return(65535&(n>>>16)+(t>>>16)+(e>>>16)+(r>>>16)+(i>>>16))<<16|65535&i}function d(n,t,e,r,i){const s=(65535&n)+(65535&t)+(65535&e)+(65535&r)+(65535&i);return(65535&(n>>>16)+(t>>>16)+(e>>>16)+(r>>>16)+(i>>>16)+(s>>>16))<<16|65535&s}function R(n){return c(n,7)^c(n,18)^a(n,3)}function U(n){return c(n,6)^c(n,11)^c(n,25)}function y(n){return[1732584193,4023233417,2562383102,271733878,3285377520]}function T(n,t){let e,r,i,s,o,w,u;const h=[];for(e=t[0],r=t[1],i=t[2],s=t[3],o=t[4],u=0;u<80;u+=1)h[u]=u<16?n[u]:f(h[u-3]^h[u-8]^h[u-14]^h[u-16],1),w=u<20?d(f(e,5),A(r,i,s),o,1518500249,h[u]):u<40?d(f(e,5),l(r,i,s),o,1859775393,h[u]):u<60?d(f(e,5),E(r,i,s),o,2400959708,h[u]):d(f(e,5),l(r,i,s),o,3395469782,h[u]),o=s,s=i,i=f(r,30),r=e,e=w;return t[0]=p(e,t[0]),t[1]=p(r,t[1]),t[2]=p(i,t[2]),t[3]=p(s,t[3]),t[4]=p(o,t[4]),t}function b(n,t,e,r){let i;const s=15+(t+65>>>9<<4),o=t+e;for(;n.length<=s;)n.push(0);for(n[t>>>5]|=128<<24-t%32,n[s]=4294967295&o,n[s-1]=o/4294967296|0,i=0;i<n.length;i+=16)r=T(n.slice(i,i+16),r);return r}class F extends h{constructor(n,t,r){if("SHA-1"!==n)throw new Error(w);super(n,t,r),this.B=-1,this.R=e(this.t,this.i,this.B),this.T=T,this.v=function(n){return n.slice()},this.Y=y,this.m=b,this.U=[1732584193,4023233417,2562383102,271733878,3285377520],this.S=512,this.F=160,this.g=!1}}function g(n){let t;return t="SHA-224"==n?s.slice():o.slice(),t}function B(n,t){let e,r,s,o,w,u,h,f,l,y,T;const b=[];for(e=t[0],r=t[1],s=t[2],o=t[3],w=t[4],u=t[5],h=t[6],f=t[7],T=0;T<64;T+=1)b[T]=T<16?n[T]:S(c(F=b[T-2],17)^c(F,19)^a(F,10),b[T-7],R(b[T-15]),b[T-16]),l=d(f,U(w),A(w,u,h),i[T],b[T]),y=p(H(e),E(e,r,s)),f=h,h=u,u=w,w=p(o,l),o=s,s=r,r=e,e=p(l,y);var F;return t[0]=p(e,t[0]),t[1]=p(r,t[1]),t[2]=p(s,t[2]),t[3]=p(o,t[3]),t[4]=p(w,t[4]),t[5]=p(u,t[5]),t[6]=p(h,t[6]),t[7]=p(f,t[7]),t}class m extends h{constructor(n,t,r){if(super(n,t,r),!1==("SHA-224"===n||"SHA-256"===n))throw new Error(w);this.B=-1,this.R=e(this.t,this.i,this.B),this.T=B,this.v=function(n){return n.slice()},this.Y=g,this.m=function(t,e,r,i){return function(n,t,e,r,i){let s,o;const w=15+(t+65>>>9<<4),u=t+e;for(;n.length<=w;)n.push(0);for(n[t>>>5]|=128<<24-t%32,n[w]=4294967295&u,n[w-1]=u/4294967296|0,s=0;s<n.length;s+=16)r=B(n.slice(s,s+16),r);return o="SHA-224"===i?[r[0],r[1],r[2],r[3],r[4],r[5],r[6]]:r,o}(t,e,r,i,n)},this.U=g(n),this.S=512,this.F="SHA-224"===n?224:256,this.g=!1}}class v{constructor(n,t){this.C=n,this.I=t}}function Y(n,t){let e;return t>32?(e=64-t,new v(n.I<<t|n.C>>>e,n.C<<t|n.I>>>e)):0!==t?(e=32-t,new v(n.C<<t|n.I>>>e,n.I<<t|n.C>>>e)):n}function C(n,t){let e;return t<32?(e=32-t,new v(n.C>>>t|n.I<<e,n.I>>>t|n.C<<e)):(e=64-t,new v(n.I>>>t|n.C<<e,n.C>>>t|n.I<<e))}function I(n,t){return new v(n.C>>>t,n.I>>>t|n.C<<32-t)}function L(n,t,e){return new v(n.C&t.C^n.C&e.C^t.C&e.C,n.I&t.I^n.I&e.I^t.I&e.I)}function M(n){const t=C(n,28),e=C(n,34),r=C(n,39);return new v(t.C^e.C^r.C,t.I^e.I^r.I)}function N(n,t){let e,r;e=(65535&n.I)+(65535&t.I),r=(n.I>>>16)+(t.I>>>16)+(e>>>16);const i=(65535&r)<<16|65535&e;return e=(65535&n.C)+(65535&t.C)+(r>>>16),r=(n.C>>>16)+(t.C>>>16)+(e>>>16),new v((65535&r)<<16|65535&e,i)}function k(n,t,e,r){let i,s;i=(65535&n.I)+(65535&t.I)+(65535&e.I)+(65535&r.I),s=(n.I>>>16)+(t.I>>>16)+(e.I>>>16)+(r.I>>>16)+(i>>>16);const o=(65535&s)<<16|65535&i;return i=(65535&n.C)+(65535&t.C)+(65535&e.C)+(65535&r.C)+(s>>>16),s=(n.C>>>16)+(t.C>>>16)+(e.C>>>16)+(r.C>>>16)+(i>>>16),new v((65535&s)<<16|65535&i,o)}function X(n,t,e,r,i){let s,o;s=(65535&n.I)+(65535&t.I)+(65535&e.I)+(65535&r.I)+(65535&i.I),o=(n.I>>>16)+(t.I>>>16)+(e.I>>>16)+(r.I>>>16)+(i.I>>>16)+(s>>>16);const w=(65535&o)<<16|65535&s;return s=(65535&n.C)+(65535&t.C)+(65535&e.C)+(65535&r.C)+(65535&i.C)+(o>>>16),o=(n.C>>>16)+(t.C>>>16)+(e.C>>>16)+(r.C>>>16)+(i.C>>>16)+(s>>>16),new v((65535&o)<<16|65535&s,w)}function K(n,t){return new v(n.C^t.C,n.I^t.I)}function x(n){const t=C(n,19),e=C(n,61),r=I(n,6);return new v(t.C^e.C^r.C,t.I^e.I^r.I)}function P(n){const t=C(n,1),e=C(n,8),r=I(n,7);return new v(t.C^e.C^r.C,t.I^e.I^r.I)}function z(n){const t=C(n,14),e=C(n,18),r=C(n,41);return new v(t.C^e.C^r.C,t.I^e.I^r.I)}const Z=[new v(i[0],3609767458),new v(i[1],602891725),new v(i[2],3964484399),new v(i[3],2173295548),new v(i[4],4081628472),new v(i[5],3053834265),new v(i[6],2937671579),new v(i[7],3664609560),new v(i[8],2734883394),new v(i[9],1164996542),new v(i[10],1323610764),new v(i[11],3590304994),new v(i[12],4068182383),new v(i[13],991336113),new v(i[14],633803317),new v(i[15],3479774868),new v(i[16],2666613458),new v(i[17],944711139),new v(i[18],2341262773),new v(i[19],2007800933),new v(i[20],1495990901),new v(i[21],1856431235),new v(i[22],3175218132),new v(i[23],2198950837),new v(i[24],3999719339),new v(i[25],766784016),new v(i[26],2566594879),new v(i[27],3203337956),new v(i[28],1034457026),new v(i[29],2466948901),new v(i[30],3758326383),new v(i[31],168717936),new v(i[32],1188179964),new v(i[33],1546045734),new v(i[34],1522805485),new v(i[35],2643833823),new v(i[36],2343527390),new v(i[37],1014477480),new v(i[38],1206759142),new v(i[39],344077627),new v(i[40],1290863460),new v(i[41],3158454273),new v(i[42],3505952657),new v(i[43],106217008),new v(i[44],3606008344),new v(i[45],1432725776),new v(i[46],1467031594),new v(i[47],851169720),new v(i[48],3100823752),new v(i[49],1363258195),new v(i[50],3750685593),new v(i[51],3785050280),new v(i[52],3318307427),new v(i[53],3812723403),new v(i[54],2003034995),new v(i[55],3602036899),new v(i[56],1575990012),new v(i[57],1125592928),new v(i[58],2716904306),new v(i[59],442776044),new v(i[60],593698344),new v(i[61],3733110249),new v(i[62],2999351573),new v(i[63],3815920427),new v(3391569614,3928383900),new v(3515267271,566280711),new v(3940187606,3454069534),new v(4118630271,4000239992),new v(116418474,1914138554),new v(174292421,2731055270),new v(289380356,3203993006),new v(460393269,320620315),new v(685471733,587496836),new v(852142971,1086792851),new v(1017036298,365543100),new v(1126000580,2618297676),new v(1288033470,3409855158),new v(1501505948,4234509866),new v(1607167915,987167468),new v(1816402316,1246189591)];function j(n){let t;return t="SHA-384"===n?[new v(3418070365,s[0]),new v(1654270250,s[1]),new v(2438529370,s[2]),new v(355462360,s[3]),new v(1731405415,s[4]),new v(41048885895,s[5]),new v(3675008525,s[6]),new v(1203062813,s[7])]:[new v(o[0],4089235720),new v(o[1],2227873595),new v(o[2],4271175723),new v(o[3],1595750129),new v(o[4],2917565137),new v(o[5],725511199),new v(o[6],4215389547),new v(o[7],327033209)],t}function q(n,t){let e,r,i,s,o,w,u,h,f,c,a,l;const A=[];for(e=t[0],r=t[1],i=t[2],s=t[3],o=t[4],w=t[5],u=t[6],h=t[7],a=0;a<80;a+=1)a<16?(l=2*a,A[a]=new v(n[l],n[l+1])):A[a]=k(x(A[a-2]),A[a-7],P(A[a-15]),A[a-16]),f=X(h,z(o),(H=w,p=u,new v((E=o).C&H.C^~E.C&p.C,E.I&H.I^~E.I&p.I)),Z[a],A[a]),c=N(M(e),L(e,r,i)),h=u,u=w,w=o,o=N(s,f),s=i,i=r,r=e,e=N(f,c);var E,H,p;return t[0]=N(e,t[0]),t[1]=N(r,t[1]),t[2]=N(i,t[2]),t[3]=N(s,t[3]),t[4]=N(o,t[4]),t[5]=N(w,t[5]),t[6]=N(u,t[6]),t[7]=N(h,t[7]),t}class D extends h{constructor(n,t,r){if(super(n,t,r),!1==("SHA-384"===n||"SHA-512"===n))throw new Error(w);this.B=-1,this.R=e(this.t,this.i,this.B),this.T=q,this.v=function(n){return n.slice()},this.Y=j,this.m=function(t,e,r,i){return function(n,t,e,r,i){let s,o;const w=31+(t+129>>>10<<5),u=t+e;for(;n.length<=w;)n.push(0);for(n[t>>>5]|=128<<24-t%32,n[w]=4294967295&u,n[w-1]=u/4294967296|0,s=0;s<n.length;s+=32)r=q(n.slice(s,s+32),r);return o="SHA-384"===i?[(r=r)[0].C,r[0].I,r[1].C,r[1].I,r[2].C,r[2].I,r[3].C,r[3].I,r[4].C,r[4].I,r[5].C,r[5].I]:[r[0].C,r[0].I,r[1].C,r[1].I,r[2].C,r[2].I,r[3].C,r[3].I,r[4].C,r[4].I,r[5].C,r[5].I,r[6].C,r[6].I,r[7].C,r[7].I],o}(t,e,r,i,n)},this.U=j(n),this.S=1024,this.F="SHA-384"===n?384:512,this.g=!1}}const G=[new v(0,1),new v(0,32898),new v(2147483648,32906),new v(2147483648,2147516416),new v(0,32907),new v(0,2147483649),new v(2147483648,2147516545),new v(2147483648,32777),new v(0,138),new v(0,136),new v(0,2147516425),new v(0,2147483658),new v(0,2147516555),new v(2147483648,139),new v(2147483648,32905),new v(2147483648,32771),new v(2147483648,32770),new v(2147483648,128),new v(0,32778),new v(2147483648,2147483658),new v(2147483648,2147516545),new v(2147483648,32896),new v(0,2147483649),new v(2147483648,2147516424)],J=[[0,36,3,41,18],[1,44,10,45,2],[62,6,43,15,61],[28,55,25,21,56],[27,20,39,8,14]];function O(n){let t;const e=[];for(t=0;t<5;t+=1)e[t]=[new v(0,0),new v(0,0),new v(0,0),new v(0,0),new v(0,0)];return e}function Q(n){let t;const e=[];for(t=0;t<5;t+=1)e[t]=n[t].slice();return e}function V(n,t){let e,r,i,s;const o=[],w=[];if(null!==n)for(r=0;r<n.length;r+=2)t[(r>>>1)%5][(r>>>1)/5|0]=K(t[(r>>>1)%5][(r>>>1)/5|0],new v(n[r+1],n[r]));for(e=0;e<24;e+=1){for(s=O(),r=0;r<5;r+=1)o[r]=(u=t[r][0],h=t[r][1],f=t[r][2],c=t[r][3],a=t[r][4],new v(u.C^h.C^f.C^c.C^a.C,u.I^h.I^f.I^c.I^a.I));for(r=0;r<5;r+=1)w[r]=K(o[(r+4)%5],Y(o[(r+1)%5],1));for(r=0;r<5;r+=1)for(i=0;i<5;i+=1)t[r][i]=K(t[r][i],w[r]);for(r=0;r<5;r+=1)for(i=0;i<5;i+=1)s[i][(2*r+3*i)%5]=Y(t[r][i],J[r][i]);for(r=0;r<5;r+=1)for(i=0;i<5;i+=1)t[r][i]=K(s[r][i],new v(~s[(r+1)%5][i].C&s[(r+2)%5][i].C,~s[(r+1)%5][i].I&s[(r+2)%5][i].I));t[0][0]=K(t[0][0],G[e])}var u,h,f,c,a;return t}class W extends h{constructor(n,t,r){let i=6,s=0;if(super(n,t,r),this.g=!1,"SHA3-224"===n)s=1152,this.F=224;else if("SHA3-256"===n)s=1088,this.F=256;else if("SHA3-384"===n)s=832,this.F=384;else if("SHA3-512"===n)s=576,this.F=512;else if("SHAKE128"===n)i=31,s=1344,this.F=-1,this.g=!0;else{if("SHAKE256"!==n)throw new Error(w);i=31,s=1088,this.F=-1,this.g=!0}this.S=s,this.B=1,this.R=e(this.t,this.i,this.B),this.T=V,this.v=Q,this.Y=O,this.m=function(n,t,e,r,o){return function(n,t,e,r,i,s,o){let w,u,h=0;const f=[],c=i>>>5,a=t>>>5;for(w=0;w<a&&t>=i;w+=c)r=V(n.slice(w,w+c),r),t-=i;for(n=n.slice(w),t%=i;n.length<c;)n.push(0);for(w=t>>>3,n[w>>2]^=s<<w%4*8,n[c-1]^=2147483648,r=V(n,r);32*f.length<o&&(u=r[h%5][h/5|0],f.push(u.I),!(32*f.length>=o));)f.push(u.C),h+=1,0==64*h%i&&(V(null,r),h=0);return f}(n,t,0,r,s,i,o)},this.U=O()}}export default class{constructor(n,t,e){if("SHA-1"==n)this.L=new F(n,t,e);else if("SHA-224"==n||"SHA-256"==n)this.L=new m(n,t,e);else if("SHA-384"==n||"SHA-512"==n)this.L=new D(n,t,e);else{if("SHA3-224"!=n&&"SHA3-256"!=n&&"SHA3-384"!=n&&"SHA3-512"!=n&&"SHAKE128"!=n&&"SHAKE256"!=n)throw new Error(w);this.L=new W(n,t,e)}}update(n){this.L.update(n)}getHash(n,t){return this.L.getHash(n,t)}setHMACKey(n,t,e){this.L.setHMACKey(n,t,e)}getHMAC(n,t){return this.L.getHMAC(n,t)}}
//# sourceMappingURL=sha.mjs.map
