/**
 * A JavaScript implementation of the SHA family of hashes, as defined in FIPS PUB 180-4 and FIPS PUB 202, as
 * well as the corresponding HMAC implementation as defined in FIPS PUB 198a
 *
 * Copyright 2008-2020 Brian Turek, 1998-2009 Paul Johnston & Contributors
 * Distributed under the BSD License
 * See http://caligatio.github.com/jsSHA/ for more information
 *
 * Two ECMAScript polyfill functions carry the following license:
 *
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
 *
 * THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED,
 * INCLUDING WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
 * MERCHANTABLITY OR NON-INFRINGEMENT.
 *
 * See the Apache Version 2.0 License for specific language governing permissions and limitations under the License.
 */
!function(r,t){"object"==typeof exports&&"undefined"!=typeof module?module.exports=t():"function"==typeof define&&define.amd?define(t):(r=r||self).jsSHA=t()}(this,(function(){"use strict";var r=function(t,n){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,t){r.__proto__=t}||function(r,t){for(var n in t)t.hasOwnProperty(n)&&(r[n]=t[n])})(t,n)};var t="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";function n(r,t,n,i){var e,o,s,u=t||[0],f=(n=n||0)>>>3,h=-1===i?3:0;for(e=0;e<r.length;e+=1)o=(s=e+f)>>>2,u.length<=o&&u.push(0),u[o]|=r[e]<<8*(h+i*(s%4));return{value:u,binLen:8*r.length+n}}function i(r,i,e){switch(i){case"UTF8":case"UTF16BE":case"UTF16LE":break;default:throw new Error("encoding must be UTF8, UTF16BE, or UTF16LE")}switch(r){case"HEX":return function(r,t,n){return function(r,t,n,i){var e,o,s,u;if(0!=r.length%2)throw new Error("String of HEX type must be in byte increments");var f=t||[0],h=(n=n||0)>>>3,a=-1===i?3:0;for(e=0;e<r.length;e+=2){if(o=parseInt(r.substr(e,2),16),isNaN(o))throw new Error("String of HEX type contains invalid characters");for(s=(u=(e>>>1)+h)>>>2;f.length<=s;)f.push(0);f[s]|=o<<8*(a+i*(u%4))}return{value:f,binLen:4*r.length+n}}(r,t,n,e)};case"TEXT":return function(r,t,n){return function(r,t,n,i,e){var o,s,u,f,h,a,c,w,A=0,E=n||[0],v=(i=i||0)>>>3;if("UTF8"===t)for(c=-1===e?3:0,u=0;u<r.length;u+=1)for(s=[],128>(o=r.charCodeAt(u))?s.push(o):2048>o?(s.push(192|o>>>6),s.push(128|63&o)):55296>o||57344<=o?s.push(224|o>>>12,128|o>>>6&63,128|63&o):(u+=1,o=65536+((1023&o)<<10|1023&r.charCodeAt(u)),s.push(240|o>>>18,128|o>>>12&63,128|o>>>6&63,128|63&o)),f=0;f<s.length;f+=1){for(h=(a=A+v)>>>2;E.length<=h;)E.push(0);E[h]|=s[f]<<8*(c+e*(a%4)),A+=1}else for(c=-1===e?2:0,w="UTF16LE"===t&&1!==e||"UTF16LE"!==t&&1===e,u=0;u<r.length;u+=1){for(o=r.charCodeAt(u),!0===w&&(o=(f=255&o)<<8|o>>>8),h=(a=A+v)>>>2;E.length<=h;)E.push(0);E[h]|=o<<8*(c+e*(a%4)),A+=2}return{value:E,binLen:8*A+i}}(r,i,t,n,e)};case"B64":return function(r,n,i){return function(r,n,i,e){var o,s,u,f,h,a,c=0,w=n||[0],A=(i=i||0)>>>3,E=-1===e?3:0,v=r.indexOf("=");if(-1===r.search(/^[a-zA-Z0-9=+/]+$/))throw new Error("Invalid character in base-64 string");if(r=r.replace(/=/g,""),-1!==v&&v<r.length)throw new Error("Invalid '=' found in base-64 string");for(o=0;o<r.length;o+=4){for(f=r.substr(o,4),u=0,s=0;s<f.length;s+=1)u|=t.indexOf(f.charAt(s))<<18-6*s;for(s=0;s<f.length-1;s+=1){for(h=(a=c+A)>>>2;w.length<=h;)w.push(0);w[h]|=(u>>>16-8*s&255)<<8*(E+e*(a%4)),c+=1}}return{value:w,binLen:8*c+i}}(r,n,i,e)};case"BYTES":return function(r,t,n){return function(r,t,n,i){var e,o,s,u,f=t||[0],h=(n=n||0)>>>3,a=-1===i?3:0;for(o=0;o<r.length;o+=1)e=r.charCodeAt(o),s=(u=o+h)>>>2,f.length<=s&&f.push(0),f[s]|=e<<8*(a+i*(u%4));return{value:f,binLen:8*r.length+n}}(r,t,n,e)};case"ARRAYBUFFER":try{new ArrayBuffer(0)}catch(r){throw new Error("ARRAYBUFFER not supported by this environment")}return function(r,t,i){return function(r,t,i,e){return n(new Uint8Array(r),t,i,e)}(r,t,i,e)};case"UINT8ARRAY":try{new Uint8Array(0)}catch(r){throw new Error("UINT8ARRAY not supported by this environment")}return function(r,t,i){return n(r,t,i,e)};default:throw new Error("format must be HEX, TEXT, B64, BYTES, ARRAYBUFFER, or UINT8ARRAY")}}function e(r,n,i,e){switch(r){case"HEX":return function(r){return function(r,t,n,i){var e,o,s="",u=t/8,f=-1===n?3:0;for(e=0;e<u;e+=1)o=r[e>>>2]>>>8*(f+n*(e%4)),s+="0123456789abcdef".charAt(o>>>4&15)+"0123456789abcdef".charAt(15&o);return i.outputUpper?s.toUpperCase():s}(r,n,i,e)};case"B64":return function(r){return function(r,n,i,e){var o,s,u,f,h,a="",c=n/8,w=-1===i?3:0;for(o=0;o<c;o+=3)for(f=o+1<c?r[o+1>>>2]:0,h=o+2<c?r[o+2>>>2]:0,u=(r[o>>>2]>>>8*(w+i*(o%4))&255)<<16|(f>>>8*(w+i*((o+1)%4))&255)<<8|h>>>8*(w+i*((o+2)%4))&255,s=0;s<4;s+=1)a+=8*o+6*s<=n?t.charAt(u>>>6*(3-s)&63):e.b64Pad;return a}(r,n,i,e)};case"BYTES":return function(r){return function(r,t,n){var i,e,o="",s=t/8,u=-1===n?3:0;for(i=0;i<s;i+=1)e=r[i>>>2]>>>8*(u+n*(i%4))&255,o+=String.fromCharCode(e);return o}(r,n,i)};case"ARRAYBUFFER":try{new ArrayBuffer(0)}catch(r){throw new Error("ARRAYBUFFER not supported by this environment")}return function(r){return function(r,t,n){var i,e=t/8,o=new ArrayBuffer(e),s=new Uint8Array(o),u=-1===n?3:0;for(i=0;i<e;i+=1)s[i]=r[i>>>2]>>>8*(u+n*(i%4))&255;return o}(r,n,i)};case"UINT8ARRAY":try{new Uint8Array(0)}catch(r){throw new Error("UINT8ARRAY not supported by this environment")}return function(r){return function(r,t,n){var i,e=t/8,o=-1===n?3:0,s=new Uint8Array(e);for(i=0;i<e;i+=1)s[i]=r[i>>>2]>>>8*(o+n*(i%4))&255;return s}(r,n,i)};default:throw new Error("format must be HEX, B64, BYTES, ARRAYBUFFER, or UINT8ARRAY")}}var o=[1116352408,1899447441,3049323471,3921009573,961987163,1508970993,2453635748,2870763221,3624381080,310598401,607225278,1426881987,1925078388,2162078206,2614888103,3248222580,3835390401,4022224774,264347078,604807628,770255983,1249150122,1555081692,1996064986,2554220882,2821834349,2952996808,3210313671,3336571891,3584528711,113926993,338241895,666307205,773529912,1294757372,1396182291,1695183700,1986661051,2177026350,2456956037,2730485921,2820302411,3259730800,3345764771,3516065817,3600352804,4094571909,275423344,430227734,506948616,659060556,883997877,958139571,1322822218,1537002063,1747873779,1955562222,2024104815,2227730452,2361852424,2428436474,2756734187,3204031479,3329325298],s=[3238371032,914150663,812702999,4144912697,4290775857,1750603025,1694076839,3204075428],u=[1779033703,3144134277,1013904242,2773480762,1359893119,2600822924,528734635,1541459225];function f(r){var t={outputUpper:!1,b64Pad:"=",shakeLen:-1},n=r||{};if(t.outputUpper=n.outputUpper||!1,n.b64Pad&&(t.b64Pad=n.b64Pad),n.shakeLen){if(n.shakeLen%8!=0)throw new Error("shakeLen must be a multiple of 8");t.shakeLen=n.shakeLen}if("boolean"!=typeof t.outputUpper)throw new Error("Invalid outputUpper formatting option");if("string"!=typeof t.b64Pad)throw new Error("Invalid b64Pad formatting option");return t}function h(r,t){return r>>>t|r<<32-t}function a(r,t){return r>>>t}function c(r,t,n){return r&t^~r&n}function w(r,t,n){return r&t^r&n^t&n}function A(r){return h(r,2)^h(r,13)^h(r,22)}function E(r,t){var n=(65535&r)+(65535&t);return(65535&(r>>>16)+(t>>>16)+(n>>>16))<<16|65535&n}function v(r,t,n,i){var e=(65535&r)+(65535&t)+(65535&n)+(65535&i);return(65535&(r>>>16)+(t>>>16)+(n>>>16)+(i>>>16)+(e>>>16))<<16|65535&e}function p(r,t,n,i,e){var o=(65535&r)+(65535&t)+(65535&n)+(65535&i)+(65535&e);return(65535&(r>>>16)+(t>>>16)+(n>>>16)+(i>>>16)+(e>>>16)+(o>>>16))<<16|65535&o}function d(r){return h(r,7)^h(r,18)^a(r,3)}function R(r){return h(r,6)^h(r,11)^h(r,25)}function y(r){return"SHA-224"==r?s.slice():u.slice()}function U(r,t){var n,i,e,s,u,f,y,U,l,b,T,F,m=[];for(n=t[0],i=t[1],e=t[2],s=t[3],u=t[4],f=t[5],y=t[6],U=t[7],T=0;T<64;T+=1)m[T]=T<16?r[T]:v(h(F=m[T-2],17)^h(F,19)^a(F,10),m[T-7],d(m[T-15]),m[T-16]),l=p(U,R(u),c(u,f,y),o[T],m[T]),b=E(A(n),w(n,i,e)),U=y,y=f,f=u,u=E(s,l),s=e,e=i,i=n,n=E(l,b);return t[0]=E(n,t[0]),t[1]=E(i,t[1]),t[2]=E(e,t[2]),t[3]=E(s,t[3]),t[4]=E(u,t[4]),t[5]=E(f,t[5]),t[6]=E(y,t[6]),t[7]=E(U,t[7]),t}return function(t){function n(r,n,e){var o=t.call(this,r,n,e)||this;if(!1==("SHA-224"===r||"SHA-256"===r))throw new Error("Chosen SHA variant is not supported");return o.t=-1,o.i=i(o.o,o.s,o.t),o.u=U,o.h=function(r){return r.slice()},o.A=y,o.v=function(t,n,i,e){return function(r,t,n,i,e){for(var o,s=15+(t+65>>>9<<4),u=t+n;r.length<=s;)r.push(0);for(r[t>>>5]|=128<<24-t%32,r[s]=4294967295&u,r[s-1]=u/4294967296|0,o=0;o<r.length;o+=16)i=U(r.slice(o,o+16),i);return"SHA-224"===e?[i[0],i[1],i[2],i[3],i[4],i[5],i[6]]:i}(t,n,i,e,r)},o.p=y(r),o.R=512,o.U="SHA-224"===r?224:256,o.l=!1,o}return function(t,n){function i(){this.constructor=t}r(t,n),t.prototype=null===n?Object.create(n):(i.prototype=n.prototype,new i)}(n,t),n}(function(){function r(r,t,n){var i=n||{};if(this.o=t,this.s=i.encoding||"UTF8",this.numRounds=i.numRounds||1,isNaN(this.numRounds)||this.numRounds!==parseInt(this.numRounds,10)||1>this.numRounds)throw new Error("numRounds must a integer >= 1");this.T=r,this.F=[],this.m=0,this.H=!1,this.B=0,this.g=!1,this.Y=[],this.S=[]}return r.prototype.update=function(r){var t,n=0,i=this.R>>>5,e=this.i(r,this.F,this.m),o=e.binLen,s=e.value,u=o>>>5;for(t=0;t<u;t+=i)n+this.R<=o&&(this.p=this.u(s.slice(t,t+i),this.p),n+=this.R);this.B+=n,this.F=s.slice(n>>>5),this.m=o%this.R,this.H=!0},r.prototype.getHash=function(r,t){var n,i,o=this.U;if(!0===this.g)throw new Error("Cannot call getHash after setting HMAC key");var s=f(t);if(!0===this.l){if(-1===s.shakeLen)throw new Error("shakeLen must be specified in options");o=s.shakeLen}var u=e(r,o,this.t,s);for(i=this.v(this.F.slice(),this.m,this.B,this.h(this.p),o),n=1;n<this.numRounds;n+=1)!0===this.l&&o%32!=0&&(i[i.length-1]&=16777215>>>24-o%32),i=this.v(i,o,0,this.A(this.T),o);return u(i)},r.prototype.setHMACKey=function(r,t,n){var e,o;if(!0===this.g)throw new Error("HMAC key already set");if(!0===this.H)throw new Error("Cannot set HMAC key after calling update");if(!0===this.l)throw new Error("SHAKE is not supported for HMAC");var s=i(t,(n||{}).encoding||"UTF8",this.t)(r),u=s.binLen,f=this.R>>>3,h=f/4-1;for(e=s.value,f<u/8&&(e=this.v(e,u,0,this.A(this.T),this.U));e.length<=h;)e.push(0);for(o=0;o<=h;o+=1)this.Y[o]=909522486^e[o],this.S[o]=1549556828^e[o];this.p=this.u(this.Y,this.p),this.B=this.R,this.g=!0},r.prototype.getHMAC=function(r,t){var n;if(!1===this.g)throw new Error("Cannot call getHMAC without first setting HMAC key");var i=f(t),o=e(r,this.U,this.t,i),s=this.v(this.F.slice(),this.m,this.B,this.h(this.p),this.U);return n=this.u(this.S,this.A(this.T)),o(n=this.v(s,this.U,this.R,n,this.U))},r}())}));
